#! /bin/bash
#
# Author Markus Gruenwald
# Url: www.mg-it.net
# Date 2008-09-08
#
#
#


PROGNAME="${0##*/}"
PROGPATH="${0%/*}"
VERSION="20141013"

UTIME=$(date +%s)

. $PROGPATH/utils.sh

NAGIOS_HOME="/usr/local/nagios"
RAMDISK="${NAGIOS_HOME}/var/tempfs"
INTERFACE_IDX_CACHE_DIR="${RAMDISK}"
INTERFACE_IDX_CACHE_MAXAGE="15" # Default-Value (min)
INTERFACE_IDX_CACHE_EXT="_if.idx"
INTERFACE_IDX_LOCK_EXT="_if.idx.lock"


STATUS=0
! SNMPWALK=$(type -p snmpwalk) && echo "ERROR: MISSING snmpwalk (part of netsnmp) not found" && exit $STATE_UNKNOWN
! SNMPGET=$(type -p snmpget) && echo "ERROR: MISSING snmpget (part of netsnmp) not found" && exit $STATE_UNKNOWN



print_usage() {
    echo ""
    echo "Usage: $PROGNAME -H <IP-address> -C <snmp community string> -e <object>"
    echo "       [-w <warning_level> -c <critical_level>]"
    echo ""
    echo "       FIRST RUN \"$PROGNAME -H <IP-address> -C <snmp community string> -e info\""
    echo "       for creating interface-index the other checks won't work."
    echo ""
    echo ""
    echo "Options:"
    echo ""
    echo " -H <IP-address>"
    echo ""
    echo " -C <snmp community string>"
    echo ""
    echo " -e <object>"
    echo ""
    echo "    available objects:"
    echo ""
    echo "    [check_cpu] (% warning and critical level possible)"
    echo ""
    echo ""
    echo "    [check_memory] (% warning and critical level possible)"
    echo ""
    echo ""
    echo "    [check_connections] (number of warning and critical connections possible)"
    echo ""
    echo ""
    echo "    [check_interface] (number of warning and critical bandwith-usage possible)"
    echo "      -i <interface-name>"
    echo "            interface-name is something like GigabitEthernet3/16"
    echo "      -x <minutes>"
    echo "            max age for interfaces index- cache-file in minutes"
    echo ""
    echo ""
    echo "    [check_fan]"
    echo "      Fan status"
    echo ""
    echo ""
    echo "    [check_power]"
    echo "      Power supply status"
    echo ""
    echo ""
    echo "    [check_temp]"
    echo "      Temperature status"
    echo ""
    echo ""
    echo "    [info]"
    echo "      System-Infos (Hostname, System-Type, Uptime)"
    echo ""
    echo ""
    echo ""
    echo " -w <warning_level> (not used by any check)"
    echo " -c <critical_level> (not used by any check)"
    echo " -E <1> activate port-status escalation (port down = return CRITICAL)"
    echo " -h, --help"
    echo " -v, --version"
    echo ""
}

print_help() {
    print_head
    echo ""
    print_usage
    echo ""
    echo "Plugin for checking Cisco Catalyst object states like cpu, memory, bandwith ..."
    echo ""
}


print_head() {
	echo "$PROGNAME Version $VERSION"
	$ECHO "This plugin come with ABSOLUTELY NO WARRANTY. You may redistribute\ncopies of the plugins under the terms of the GNU General Public License.\n"
}


validate_levels() {

	if [[ "$W_LEVEL" == "" ]] && [[ "$C_LEVEL" != "" ]]
	  then
	     echo " both (WARNING and CRITICAL) levels must be set"
	     exit $STATE_UNKNOWN
	elif [[ "$W_LEVEL" != "" ]] && [[ "$C_LEVEL" == "" ]]
	  then
	     echo " both (WARNING and CRITICAL) levels must be set"
	     exit $STATE_UNKNOWN
	fi


	for E in $W_LEVEL $C_LEVEL
	do
	  if [[ "$E" != "" ]] && expr $E : "[0-9]*\$" > /dev/null
	    then
	      if [[ "$1" == "p" ]] && ! (( $E <= 100 ))  && ! (( $E >= 0 ))
	        then
	          LEVEL_STATE=1
	          echo " WARNING and CRITICAL level must be percentage value between 0 and 100"
	          exit $STATE_UNKNOWN

	        else
	          LEVEL_STATE=0
	      fi
	  else
	    LEVEL_STATE=1
	    echo " WARNING and CRITICAL level must be numeric"
	    exit $STATE_UNKNOWN

	  fi
	done

	if [[ "$LEVEL_STATE" == "0" ]] && (( $W_LEVEL > $C_LEVEL ))
	  then
	    LEVEL_STATE=1
	    echo " WARNING level cannot be larger than CRITICAL level"
	    exit $STATE_UNKNOWN

	fi

}


set_exit_state() {

	VAL=$1
	  if [[ "$W_LEVEL" != "" ]] && [[ "$C_LEVEL" != "" ]]
	      then
		  if (( $VAL <  $W_LEVEL ))
		    then
		      RETVAL=$STATE_OK
		  
		  elif (( $VAL >=  $W_LEVEL )) && (( $VAL <  $C_LEVEL ))
		    then
		      RETVAL=$STATE_WARNING
		  
		  elif (( $VAL >  $C_LEVEL ))
		    then
		      RETVAL=$STATE_CRITICAL
		  
		  fi
	fi
}


create_if_index () {
		touch $INTERFACE_IDX_LOCK
		$SNMPWALK -v1 $HOSTADDR -c $COMMUNITY -O qs "IF-MIB::ifDescr" >> $INTERFACE_IDX_LOCK
		$SNMPWALK -v1 $HOSTADDR -c $COMMUNITY -O qs "IF-MIB::ifAlias" >> $INTERFACE_IDX_LOCK
		$SNMPWALK -v1 $HOSTADDR -c $COMMUNITY -O qs "IP-MIB::ipAdEntIfIndex" >> $INTERFACE_IDX_LOCK
		$SNMPWALK -v1 $HOSTADDR -c $COMMUNITY -O qs "CISCO-SMI::ciscoMgmt.68.1.2.2.1.2" >> $INTERFACE_IDX_LOCK
		$SNMPWALK -v1 $HOSTADDR -c $COMMUNITY -O qs "CISCO-SMI::ciscoMgmt.23.1.2.1.1.6" >> $INTERFACE_IDX_LOCK
		$SNMPWALK -v1 $HOSTADDR -c $COMMUNITY -O qs "CISCO-SMI::ciscoMgmt.23.1.2.1.1.7" >> $INTERFACE_IDX_LOCK
		$SNMPWALK -v1 $HOSTADDR -c $COMMUNITY -O qs "ENTITY-MIB::entPhysicalClass" >> $INTERFACE_IDX_LOCK
		mv $INTERFACE_IDX_LOCK $INTERFACE_IDX_CACHE
}


check_memory_status() {

	SNMP_OUT=$($SNMPGET -v 2c -O vq $HOSTADDR -c $COMMUNITY .1.3.6.1.4.1.9.9.109.1.1.1.1.12.1 .1.3.6.1.4.1.9.9.109.1.1.1.1.13.1 2>/dev/null)
	
	read MEM_USED MEM_FREE MEM_USED_UNIT MEM_FREE_UNIT <<< $SNMP_OUT
	
	
	MEM_TOTAL=$(($MEM_USED+$MEM_FREE))
	MEM_USAGE=$(($MEM_USED*100/$MEM_TOTAL))

	RETSTR="Memory usage: $MEM_USAGE% - memory used: $MEM_USED $MEM_USED_UNIT, memory free: $MEM_FREE $MEM_FREE_UNIT"
	PERF_DATA="'Memory usage'=$MEM_USAGE%;$W_LEVEL;$C_LEVEL"

	validate_levels p
	
	set_exit_state $MEM_USAGE
	
}


check_cpu_status() {
	
	CPU_USAGE=$($SNMPGET -v1 $HOSTADDR -c $COMMUNITY -O vq CISCO-PROCESS-MIB::cpmCPUTotal5minRev.1)
	RETSTR="5 minute average of the CPU busy percentage: $CPU_USAGE%"
	PERF_DATA="'5 minute average of the CPU busy percentage'=$CPU_USAGE%;$W_LEVEL;$C_LEVEL"

	validate_levels p
	
	set_exit_state $CPU_USAGE

}


check_interface_counters() {
	RETVAL=$STATE_OK

	if INTERFACE_ID=$(grep " ${IF_DESCR}$" $INTERFACE_IDX_CACHE)
	  then
	    INTERFACE_ID=${INTERFACE_ID% *}
	    INTERFACE_ID=${INTERFACE_ID##*.}
	
	  else
	    echo "Interface unknown!<br><b>$IF_DESCR</b> not found in IF-MIB::ifDescr on host $HOSTADDR"
	    exit $STATE_UNKNOWN
	    
	fi


	if IF_ALIAS=$(grep "ifAlias.${INTERFACE_ID}" $INTERFACE_IDX_CACHE)
	  then
	    IF_ALIAS=${IF_ALIAS/ifAlias.${INTERFACE_ID}/}
	
	  else
	    IF_ALIAS=""
	    
	fi


	




	IF_INFOS=$($SNMPGET -v1 $HOSTADDR -c $COMMUNITY -O vq0 IF-MIB::ifOperStatus.$INTERFACE_ID IF-MIB::ifAdminStatus.$INTERFACE_ID IF-MIB::ifPhysAddress.$INTERFACE_ID CISCO-SMI::ciscoMgmt.151.1.1.1.1.1.$INTERFACE_ID 2> /dev/null)
	
	read IF_OPER_STATUS IF_ADMIN_STATUS	IF_PHYS_ADDRESS IS_SWITCHPORT <<< $IF_INFOS
	

	if [[ "$IS_SWITCHPORT" == "1" ]]
	  then
	    IF_MOD="routed"
	    
	    IF_IP_ADDRESS=$(grep "ipAdEntIfIndex" $INTERFACE_IDX_CACHE |grep " ${INTERFACE_ID}$"|cut -d " " -f1|cut -d "." -f2-)
	    IF_IP_ADDRESS_MSG="<br /><b>IP-Address: $IF_IP_ADDRESS</b>"
	elif [[ "$IS_SWITCHPORT" == "2" ]]
	  then
	    IF_MOD="switched"

	fi

	if [[ "$IF_ADMIN_STATUS" == "up" ]] && [[ "$IF_MOD" == "switched" ]]
	  then

	    if CDP_DEVICE=$(grep "ciscoMgmt.23.1.2.1.1.6.$INTERFACE_ID" $INTERFACE_IDX_CACHE)
	      then
	        CDP_DEVICE=${CDP_DEVICE##* }
	        CDP_DEVICE=${CDP_DEVICE//\"/}
	        CDP_DEVICE=${CDP_DEVICE//.mainz.de/}
	        CDP_INTERFACE=$(grep "ciscoMgmt.23.1.2.1.1.7.$INTERFACE_ID" $INTERFACE_IDX_CACHE)
	        CDP_INTERFACE=${CDP_INTERFACE##* }
	        CDP_INTERFACE=${CDP_INTERFACE//\"/}
	        CDP_INFO_MSG="<br />Uplink to: <a href='status.cgi?host=${CDP_DEVICE}'>${CDP_DEVICE}</a>/<a href='extinfo.cgi?type=2&host=${CDP_DEVICE}&service=Interface_$(echo ${CDP_INTERFACE}|tr / _)'>${CDP_INTERFACE}</a>"
	        PORT_INFO_MSG="$CDP_INFO_MSG"
      fi

	    VLAN_STATUS=$(grep "ciscoMgmt.68.1.2.2.1.2.${INTERFACE_ID} " $INTERFACE_IDX_CACHE)
	    VLAN_STATUS=${VLAN_STATUS##* }

	    grep -q "Port-channel" <<< $IF_DESCR && IS_PORT_CH="yes"

	    if [[ "$VLAN_STATUS" == "" ]] && [[ "$IS_PORT_CH" != "yes" ]]
	      then
	        SWITCHPORT_MODE_MSG=", MODE:trunk"

	        PORT_CHANNEL_MEMBER=$(grep "ciscoMgmt.98.1.1.1.1.5.${INTERFACE_ID} " $INTERFACE_IDX_CACHE)
	        PORT_CHANNEL_MEMBER=${PORT_CHANNEL_MEMBER##* }
	        if [[ "$PORT_CHANNEL_MEMBER" != "" ]] && [[ "$PORT_CHANNEL_MEMBER" != "0" ]]
	          then
	            SYS_NAME=$($SNMPGET -v1 $HOSTADDR -c $COMMUNITY "SNMPv2-MIB::sysName.0" -O vq)
	            SYS_NAME=${SYS_NAME//.mainz.de/}
	            PORT_CHANNEL_INFO="<br />Member of <a href='extinfo.cgi?type=2&host=${SYS_NAME}&service=Interface_Port-channel${PORT_CHANNEL_MEMBER}'>Port-channel$PORT_CHANNEL_MEMBER</a>"

	        fi

	        PORT_INFO_MSG="$CDP_INFO_MSG$PORT_CHANNEL_INFO"

	    elif [[ "$IS_PORT_CH" == "yes" ]]
	      then
	        PORT_CHANNEL_MEMBERS=$(for PORT_ID in $(grep "ciscoMgmt.98.1.1.1.1.5" $INTERFACE_IDX_CACHE|grep " ${IF_DESCR/Port-channel/}$"|cut -d " " -f1)
	          do
	          	PORT_ID=${PORT_ID##*.}
	            CHANNEL_MEMBER=$(grep "ifDescr.$PORT_ID " $INTERFACE_IDX_CACHE)
	            CHANNEL_MEMBER=${CHANNEL_MEMBER##* }
	            echo $CHANNEL_MEMBER
	          done)
	        
	        SYS_NAME=$($SNMPGET -v1 $HOSTADDR -c $COMMUNITY "SNMPv2-MIB::sysName.0" -O vq)
	        SYS_NAME=${SYS_NAME//.mainz.de/}
	        for PORT_CHANNEL_IF in $PORT_CHANNEL_MEMBERS
	          do
	            PORT_CHANNEL_MEMBER_ID=$(grep " ${PORT_CHANNEL_IF}$" $INTERFACE_IDX_CACHE)
	            PORT_CHANNEL_MEMBER_ID=${PORT_CHANNEL_MEMBER_ID% *}
	            PORT_CHANNEL_MEMBER_ID=${PORT_CHANNEL_MEMBER_ID##*.}

	            PCM_INFOS=$($SNMPGET -v1 $HOSTADDR -c $COMMUNITY -O vq IF-MIB::ifOperStatus.$PORT_CHANNEL_MEMBER_ID IF-MIB::ifAdminStatus.$PORT_CHANNEL_MEMBER_ID)

	            read PCM_OPER_STATUS PCM_ADMIN_STATUS <<< $PCM_INFOS

	            if [[ "$PCM_OPER_STATUS" == "down" ]]
	              then
	                PCM_OPER_COL_TAG="style='color:red'"
	                [[ "$ESCALATION" == "1" ]] && [[ "$PCM_ADMIN_STATUS" == "up" ]] && PCM_STATUS=2

	            elif [[ "$PCM_OPER_STATUS" == "up" ]]
	              then
	                PCM_OPER_COL_TAG="style='color:green'"

	            fi

	            MEMBER_URL_LIST="$MEMBER_URL_LIST <a $PCM_OPER_COL_TAG href='extinfo.cgi?type=2&host=${SYS_NAME}&service=Interface_${PORT_CHANNEL_IF//\//_}'>$PORT_CHANNEL_IF</a>"
	          done
	        
	        PORT_INFO_MSG="<br />$IF_DESCR members:$MEMBER_URL_LIST"

	      else
		  SWITCHPORT_MODE_MSG=", VLAN$VLAN_STATUS"

	    fi

	fi


	if [[ "$IF_ADMIN_STATUS" == "down" ]]
	  then
	    echo "**************** 1" >/dev/null
	    HTML_START_TAG="<span style='color:white'>"
	    HTML_END_TAG="</span>"

	elif [[ "$IS_PORT_CH" == "yes" ]] && [[ "$IF_ADMIN_STATUS" == "up" ]] &&  [[ "$PCM_STATUS" == "2" ]]
	  then
	    echo "**************** 2" >/dev/null
	    IF_OPER_COL_START_TAG="<span style='color:red'><b>"
	    IF_OPER_COL_END_TAG="</b></span>"
	    
	    [[ "$ESCALATION" == "1" ]] && RETVAL=$STATE_CRITICAL

	elif [[ $PORT_CHANNEL_MEMBER != "" ]] && [[ $PORT_CHANNEL_MEMBER != "0" ]] && [[ "$IF_ADMIN_STATUS" == "up" ]] && [[ "$IF_OPER_STATUS" == "down" ]]
	  then
	    echo "**************** 3" >/dev/null
	    IF_OPER_COL_START_TAG="<span style='color:red'><b>"
	    IF_OPER_COL_END_TAG="</b></span>"
	    
	    [[ "$ESCALATION" == "1" ]] && RETVAL=$STATE_CRITICAL

	elif [[ "$IF_ADMIN_STATUS" == "up" ]] && [[ "$IF_OPER_STATUS" == "down" ]]
	  then
	    echo "**************** 4" >/dev/null
	    IF_OPER_COL_START_TAG="<span style='color:red'><b>"
	    IF_OPER_COL_END_TAG="</b></span>" 
	    
	    [[ "$ESCALATION" == "1" ]] && RETVAL=$STATE_OK

	elif [[ "$IF_ADMIN_STATUS" == "up" ]] && [[ "$IF_OPER_STATUS" == "up" ]]
	  then
	    echo "**************** 5" >/dev/null
	    IF_OPER_COL_START_TAG="<span style='color:green'><b>"
	    IF_OPER_COL_END_TAG="</b></span>" 

	fi


	if [[ "$IF_OPER_STATUS" == "up" ]]
	  then

	    IF_STATISTICS=$($SNMPGET -v1 $HOSTADDR -c $COMMUNITY -O vq0 IF-MIB::ifSpeed.$INTERFACE_ID IF-MIB::ifInOctets.$INTERFACE_ID IF-MIB::ifInErrors.$INTERFACE_ID IF-MIB::ifInDiscards.$INTERFACE_ID IF-MIB::ifInBroadcastPkts.$INTERFACE_ID IF-MIB::ifInMulticastPkts.$INTERFACE_ID IF-MIB::ifOutOctets.$INTERFACE_ID IF-MIB::ifOutErrors.$INTERFACE_ID IF-MIB::ifOutDiscards.$INTERFACE_ID IF-MIB::ifOutBroadcastPkts.$INTERFACE_ID IF-MIB::ifOutMulticastPkts.$INTERFACE_ID)
	    
	    read IF_SPEED IF_IN_OCTETS IF_IN_ERRORS IF_IN_DISCARDS IF_IN_BROADCASTPKTS IF_IN_MULTICASTPKTS IF_OUT_OCTETS IF_OUT_ERRORS IF_OUT_DISCARDS IF_OUT_BROADCASTPKTS IF_OUT_MULTICASTPKTS <<< $IF_STATISTICS

	    IF_IN_BIT=$(($IF_IN_OCTETS*8))
	    IF_OUT_BIT=$(($IF_OUT_OCTETS*8))

	fi


	IF_INFO_MSG="${IF_MOD}${SWITCHPORT_MODE_MSG}"
	[[ "$IF_INFO_MSG" != "" ]] && IF_INFO_MSG="(${IF_INFO_MSG})"

	RETSTR="$HTML_START_TAG$IF_DESCR ${IF_INFO_MSG}<br />Description: ${IF_ALIAS}${IF_IP_ADDRESS_MSG}<br />MAC-Address: $IF_PHYS_ADDRESS<br />Admin state: $IF_ADMIN_STATUS<br />Oper state: ${IF_OPER_COL_START_TAG}${IF_OPER_STATUS}${IF_OPER_COL_END_TAG}$PORT_INFO_MSG$HTML_END_TAG"
	PERF_DATA="'inBit (Bit)'="$IF_IN_BIT"c;;;0;$IF_SPEED 'outBit (Bit)'="$IF_OUT_BIT"c;;;0;$IF_SPEED inErrors="$IF_IN_ERRORS"c;;;; outErrors="$IF_OUT_ERRORS"c;;;; inDiscards="$IF_IN_DISCARDS"c;;;; outDiscards="$IF_OUT_DISCARDS"c;;;; inBroadcastPkts="$IF_IN_BROADCASTPKTS"c;;;; outBroadcastPkts="$IF_OUT_BROADCASTPKTS"c;;;; inMulticastPkts="$IF_IN_MULTICASTPKTS"c;;;; outMulticastPkts="$IF_OUT_MULTICASTPKTS"c;;;;"
	
}


system_info() {


find ${INTERFACE_IDX_CACHE_DIR} -name "*.${INTERFACE_IDX_LOCK_EXT}" -mmin +3 -exec rm -rf {} \;

if [ ! -f $INTERFACE_IDX_CACHE ] && [ ! -f $INTERFACE_IDX_LOCK ]
	  then
	    create_if_index

elif [ ! -f $INTERFACE_IDX_CACHE ] && ls ${INTERFACE_IDX_CACHE_DIR}/*.lock > /dev/null # [ -f $INTERFACE_IDX_LOCK ]
  then
    sleep 1
    echo "pending ..."
    exit $STATE_OK

elif [ -f $INTERFACE_IDX_CACHE ] && [ ! -f $INTERFACE_IDX_LOCK ]
  then

	  if find $INTERFACE_IDX_CACHE_DIR -maxdepth 1 -name ${INTERFACE_IDX_CACHE##*/} -mmin +$INTERFACE_IDX_CACHE_MAXAGE|grep $INTERFACE_IDX_CACHE > /dev/null && [ ! -f $INTERFACE_IDX_LOCK ]
	    then
	      create_if_index
	  fi

fi

	
	SNMPOUT=$($SNMPGET -v 2c $HOSTADDR -c $COMMUNITY -O vq SNMPv2-MIB::sysName.0 SNMPv2-MIB::sysUpTime.0 CISCO-SYSTEM-EXT-MIB::cseSysConfLastChange.0 CISCO-SYSTEM-EXT-MIB::cseSysUpTime.0)
	
	read SYS_NAME SYS_UPTIME  SYS_CONF_LAST_CHANGE SYS_UPSECONDS<<< $SNMPOUT
	
	SYS_DESCR=$($SNMPGET -v 2c $HOSTADDR -c $COMMUNITY -O vq SNMPv2-MIB::sysDescr.0|cut -d , -f1-3)
	
	RETSTR="Hostname: $SYS_NAME<br />$SYS_DESCR<br />Uptime: $SYS_UPTIME ($SYS_UPSECONDS)<br />Last config change: $SYS_CONF_LAST_CHANGE"
	RETVAL=$STATE_OK

}


check_temp() {

	RETVAL=0

	SENSOR_ID_LIST=$(grep "sensor" $INTERFACE_IDX_CACHE|cut -d " " -f1|cut -d "." -f2-)

	OID_LIST=$(
	  for ID in $SENSOR_ID_LIST
	    do
	      echo "ENTITY-MIB::entPhysicalDescr.$ID CISCO-SMI::ciscoMgmt.91.1.1.1.1.4.$ID"
	    done
	    )
	
	
	SNMPOUT=$($SNMPGET -v2c $HOSTADDR -c $COMMUNITY -O qs $OID_LIST)
	SENSOR_THRESHOLD_LIST=$($SNMPWALK -v2c $HOSTADDR -O qs -c $COMMUNITY CISCO-SMI::ciscoMgmt.91.1.2.1.1.4)
	
	IFS_BAK=$IFS
	IFS=$'\n'
	
		for VAL in $SNMPOUT
		  do
		    TEMP_ARRAY+=($VAL)
		  done
	
		for VAL in $SENSOR_THRESHOLD_LIST
		  do
		    THRESHOLD_LIST_ARRAY+=($VAL)
		  done
	
	IFS=$IFS_BAK
	
	for (( i=0; i<${#TEMP_ARRAY[@]}; i++ ))
	  do
	    TMP=$(cut -d " " -f1 <<< ${TEMP_ARRAY[$i]})
	    VAR=${TMP%.*}
	    ID=${TMP##*.}
	    VAL=$(cut -d " " -f2- <<< ${TEMP_ARRAY[$i]})
	    if [[ "$VAR" == "entPhysicalDescr" ]]
	      then
	        MOD_DESC[$ID]=$VAL
	    
	    elif [[ "$VAR" == "ciscoMgmt.91.1.1.1.1.4" ]]
	      then
	        SENSOR_TEMP[$ID]=$VAL
	    fi
	  done
	
	for (( i=0; i<${#THRESHOLD_LIST_ARRAY[@]}; i++ ))
	  do
	    TMP=$(cut -d . -f 8- <<< ${THRESHOLD_LIST_ARRAY[$i]})
	    VAL=${TMP##*" "}
	    TMP=${TMP%" "*}
	    ID=${TMP%.*}
	    SEVERITY_LEVEL=${TMP##*.}
	    if [[ "$SEVERITY_LEVEL" == "1" ]]
	      then
	        WARN_TH[$ID]=$VAL
	     
	    elif [[ "$SEVERITY_LEVEL" == "2" ]]
	      then
	        CRIT_TH[$ID]=$VAL
	    fi    
	  done
	
	
	
	
	for ID in $SENSOR_ID_LIST
	  do
	    if (( ${SENSOR_TEMP[$ID]} >= ${WARN_TH[$ID]} ))
	      then
	        (( $RETVAL < $STATE_WARNING )) && RETVAL=$STATE_WARNING
	        STRING="<br />${MOD_DESC[$ID]}: ${SENSOR_TEMP[$ID]}&deg;C<b> - WARNING (${WARN_TH[$ID]}/${CRIT_TH[$ID]})</b>"
	    
	    elif (( ${SENSOR_TEMP[$ID]} >= ${CRIT_TH[$ID]} ))
	      then
	        RETVAL=$STATE_CRITICAL
	        STRING="<br /><b>${MOD_DESC[$ID]}: ${SENSOR_TEMP[$ID]}&deg;C - CRITICAL (${WARN_TH[$ID]}/${CRIT_TH[$ID]})</b>"
	      
	      else
	        STRING="<br />${MOD_DESC[$ID]}: ${SENSOR_TEMP[$ID]}&deg;C - OK (${WARN_TH[$ID]}/${CRIT_TH[$ID]})"
	    fi
	
	    
	    MSG="${MSG}${STRING}"
	
	  done


RETSTR="Temperature status:<br />Sensor: Current &deg;C - Status (Warning/Critical) $MSG"

}


check_fan() {

	RETVAL=0
	STATUS="OK"
	
	FAN_ID_LIST=$(grep "fan" $INTERFACE_IDX_CACHE|cut -d " " -f1|cut -d "." -f2-)
	
	OID_LIST=$(
	for ID in $FAN_ID_LIST
	  do
	    echo "ENTITY-MIB::entPhysicalDescr.$ID ENTITY-MIB::entPhysicalModelName.$ID CISCO-ENTITY-FRU-CONTROL-MIB::cefcFanTrayOperStatus.$ID"
	  done
	  )
	
	
	SNMPOUT=$($SNMPGET -v2c $HOSTADDR -c $COMMUNITY -O qs $OID_LIST)
	LED_STATUS=$($SNMPWALK -v2c $HOSTADDR -c $COMMUNITY .1.3.6.1.4.1.9.9.195.1.3.1.2 -O qs|cut -d . -f 7-)
	
	IFS_BAK=$IFS
	IFS=$'\n'
	
	for VAL in $SNMPOUT
	  do
	    FAN_ARRAY+=($VAL)
	  done
	
	for VAL in $LED_STATUS
	  do
	    LED_STATUS_ARRAY+=($VAL)
	  done
	
	IFS=$IFS_BAK
	
	
	for (( i=0; i<${#FAN_ARRAY[@]}; i++ ))
	  do
	   LINE=${FAN_ARRAY[$i]}
	   TMP=$(cut -d " " -f1 <<< ${LINE})
	   VAR=${TMP%.*}
	   ID=${TMP##*.}
	   VAL=$(cut -d " " -f2- <<< ${LINE})
	   if [[ "$VAR" == "entPhysicalDescr" ]]
	     then
	       MOD_DESC[$ID]=$VAL
	
	   elif [[ "$VAR" == "entPhysicalModelName" ]]
	     then
	       MOD_NAME[$ID]=$VAL
	
	   elif [[ "$VAR" == "cefcFanTrayOperStatus" ]]
	     then
	       MOD_OPER_STAT[$ID]=$VAL
	   fi

	  done
	
	for (( i=0; i<${#LED_STATUS_ARRAY[@]}; i++ ))
	  do
	   LINE=${LED_STATUS_ARRAY[$i]}
	   ID=${LINE%.*}
	   VAL=${LINE##*" "}
	   MOD_LED_STAT[$ID]=$VAL
	  done

	for ID in $FAN_ID_LIST
	  do 
	    ! grep -q "FanModule" <<< ${MOD_DESC[$ID]} && continue
	    [[ "${MOD_LED_STAT[$ID]}" != "2" ]] && RETVAL=$STATE_CRITICAL && STATUS="CRITICAL"
	    if grep -qi "fex" <<< ${MOD_DESC[$ID]}
	      then
	        STRING="${MOD_DESC[$ID]} is ${MOD_OPER_STAT[$ID]}"
	      
	      else
	        STRING="${MOD_NAME[$ID]}, ${MOD_DESC[$ID]} is ${MOD_OPER_STAT[$ID]}"
	    fi
	
	    if [[ "${MOD_LED_STAT[$ID]}" != "2" ]]
	      then                                
	        RETVAL=$STATE_CRITICAL
	        STRING="<br /><b>${STRING} - CRITICAL</b>"
	
	      else
	        STRING="<br />${STRING} - OK"
	    fi
	    
	    MSG="${MSG}${STRING}"
	    
	  done
	
	
	RETSTR="Fan status:$MSG"

}


check_power() {
	
	
	
	
	RETVAL=0
	
	PWR_ID_LIST=$(grep "powerSupply" $INTERFACE_IDX_CACHE|cut -d " " -f1|cut -d "." -f2-)
	
	
	
	OID_LIST=$(
	  for ID in $PWR_ID_LIST
	    do
	      echo "ENTITY-MIB::entPhysicalDescr.$ID ENTITY-MIB::entPhysicalModelName.$ID CISCO-ENTITY-FRU-CONTROL-MIB::cefcFRUPowerOperStatus.$ID"
	    done
	    )
	
	
	SNMPOUT=$($SNMPGET -v2c $HOSTADDR -c $COMMUNITY -O qs $OID_LIST)
	LED_STATUS=$($SNMPWALK -v2c $HOSTADDR -c $COMMUNITY .1.3.6.1.4.1.9.9.195.1.3.1.2 -O qs|cut -d . -f 7-)
	
	IFS_BAK=$IFS
	IFS=$'\n'
	
	for VAL in $SNMPOUT
	  do
	    POWER_ARRAY+=($VAL)
	  done
	
	for VAL in $LED_STATUS
	  do
	    LED_STATUS_ARRAY+=($VAL)
	  done
	
	
	IFS=$IFS_BAK
	
	for (( i=0; i<${#POWER_ARRAY[@]}; i++ ))
	  do
	   LINE=${POWER_ARRAY[$i]}
	   TMP=$(cut -d " " -f1 <<< ${LINE})
	   VAR=${TMP%.*}
	   ID=${TMP##*.}
	   VAL=$(cut -d " " -f2- <<< ${LINE})
	   if [[ "$VAR" == "entPhysicalDescr" ]]
	     then
	       MOD_DESC[$ID]=$VAL
	
	   elif [[ "$VAR" == "entPhysicalModelName" ]]
	     then
	       MOD_NAME[$ID]=$VAL
	
	   elif [[ "$VAR" == "cefcFRUPowerOperStatus" ]]
	     then
	       MOD_OPER_STAT[$ID]=$VAL
	   fi
	  done
	
	for (( i=0; i<${#LED_STATUS_ARRAY[@]}; i++ ))
	  do
	   LINE=${LED_STATUS_ARRAY[$i]}
	   ID=${LINE%.*}
	   VAL=${LINE##*" "}
	   MOD_LED_STAT[$ID]=$VAL
	  done
	
	
	C=0
	for ID in $PWR_ID_LIST
	  do
	    C=$[$C+1]
	    if [[ "${MOD_LED_STAT[$ID]}" != "2" ]]
	      then
	        RETVAL=$STATE_CRITICAL
	        STRING="<br /><b>$C: ${MOD_DESC[$ID]:0:28} is ${MOD_OPER_STAT[$ID]} - CRITICAL</b>"
	      
	      else
	        STRING="<br />$C: ${MOD_DESC[$ID]:0:28} is ${MOD_OPER_STAT[$ID]} - OK"
	    fi
	
	
	    MSG="${MSG}${STRING}"
	
	  done
	
	
	
	RETSTR="Power supply status:$MSG"
	
}


check_chassis() {
	
	RETVAL=0
	
	CHASSIS_ID_LIST=$(grep "chassis" $INTERFACE_IDX_CACHE|cut -d " " -f1|cut -d "." -f2-)
	MODULE_ID_LIST=$(grep "module" $INTERFACE_IDX_CACHE|cut -d " " -f1|cut -d "." -f2-)
	
	
	LED_STATUS=$($SNMPWALK -v2c $HOSTADDR -c $COMMUNITY .1.3.6.1.4.1.9.9.195.1.3.1.2 -O qs|cut -d . -f 7-)
	
	IFS_BAK=$IFS
	IFS=$'\n'
	
	for VAL in $LED_STATUS
	  do
	    LED_STATUS_ARRAY+=($VAL)
	  done
	
	IFS=$IFS_BAK
	
	
	OID_LIST=$(
	  for ID in $CHASSIS_ID_LIST
	    do
	      echo "ENTITY-MIB::entPhysicalDescr.$ID ENTITY-MIB::entPhysicalModelName.$ID CISCO-ENTITY-FRU-CONTROL-MIB::cefcModuleOperStatus.$ID ENTITY-MIB::entPhysicalSerialNum.$ID"
	    done
	    )
	
	SNMPOUT=$($SNMPGET -v2c $HOSTADDR -c $COMMUNITY -O qs $OID_LIST)
	
	IFS_BAK=$IFS
	IFS=$'\n'
	
	for VAL in $SNMPOUT
	  do
	    CHASSIS_ARRAY+=($VAL)
	  done
	
	IFS=$IFS_BAK
	
	
	for (( i=0; i<${#CHASSIS_ARRAY[@]}; i++ ))
	  do
	   LINE=${CHASSIS_ARRAY[$i]}
	   TMP=$(cut -d " " -f1 <<< ${LINE})
	   VAR=${TMP%.*}
	   ID=${TMP##*.}
	   VAL=$(cut -d " " -f2- <<< ${LINE})
	   if [[ "$VAR" == "entPhysicalDescr" ]]
	     then
	       MOD_DESC[$ID]=$VAL
	
	   elif [[ "$VAR" == "entPhysicalModelName" ]]
	     then
	       MOD_NAME[$ID]=$VAL
	
	   elif [[ "$VAR" == "cefcModuleOperStatus" ]]
	     then
	       MOD_OPER_STAT[$ID]=$(tr "[:lower:]" "[:upper:]" <<< $VAL)
	
	   elif [[ "$VAR" == "entPhysicalSerialNum" ]]
	     then
	       MOD_SERIAL_NUM[$ID]=$VAL
	   fi
	  done


	OID_LIST=$(
	  for ID in $MODULE_ID_LIST
	    do
	      echo "ENTITY-MIB::entPhysicalDescr.$ID ENTITY-MIB::entPhysicalModelName.$ID CISCO-ENTITY-FRU-CONTROL-MIB::cefcFRUPowerAdminStatus.$ID CISCO-ENTITY-FRU-CONTROL-MIB::cefcFRUPowerOperStatus.$ID CISCO-ENTITY-FRU-CONTROL-MIB::cefcModuleOperStatus.$ID ENTITY-MIB::entPhysicalSerialNum.$ID"
	    done
	    )
	
	SNMPOUT=$($SNMPGET -v2c $HOSTADDR -c $COMMUNITY -O qs $OID_LIST)
	
	IFS_BAK=$IFS
	IFS=$'\n'
	
	for VAL in $SNMPOUT
	  do
	    MODULE_ARRAY+=($VAL)
	  done
	
	IFS=$IFS_BAK
	
	
	
	for (( i=0; i<${#MODULE_ARRAY[@]}; i++ ))
	  do
	   LINE=${MODULE_ARRAY[$i]}
	   TMP=$(cut -d " " -f1 <<< ${LINE})
	   VAR=${TMP%.*}
	   ID=${TMP##*.}
	   VAL=$(cut -d " " -f2- <<< ${LINE})
	   if [[ "$VAR" == "entPhysicalDescr" ]]
	     then
	       MOD_DESC[$ID]=$VAL
	
	   elif [[ "$VAR" == "entPhysicalModelName" ]]
	     then
	       MOD_NAME[$ID]=$VAL
	
	   elif [[ "$VAR" == "cefcFRUPowerAdminStatus" ]]
	     then
	       PWR_ADMIN_STAT[$ID]=$VAL
	
	   elif [[ "$VAR" == "cefcFRUPowerOperStatus" ]]
	     then
	       PWR_OPER_STAT[$ID]=$VAL
	
	   elif [[ "$VAR" == "cefcModuleOperStatus" ]]
	     then
	       MOD_OPER_STAT[$ID]=$(tr "[:lower:]" "[:upper:]" <<< $VAL)
	
	   elif [[ "$VAR" == "entPhysicalSerialNum" ]]
	     then
	       MOD_SERIAL_NUM[$ID]=$VAL
	   fi
	  done
	
	
	for ID in $CHASSIS_ID_LIST
	  do
	
	    unset NAME
	    unset OPSTAT
	    ! grep -q "Fex" <<< ${MOD_DESC[$ID]} && NAME="${MOD_NAME[$ID]}, "
	    ! grep -qi "No Such Instance" <<< ${MOD_OPER_STAT[$ID]} && OPSTAT=" - ${MOD_OPER_STAT[$ID]}"
	    INFO_STRING="(${NAME}S/N: ${MOD_SERIAL_NUM[$ID]})"
	    
	    if [[ "${MOD_OPER_STAT[$ID]}" != "" ]] && ! grep -qi "No Such Instance" <<< ${MOD_OPER_STAT[$ID]} && [[ "${MOD_OPER_STAT[$ID]}" != "OK" ]]
	      then
	        RETVAL=$STATE_CRITICAL
	        STRING="<br /><b>${MOD_DESC[$ID]} ${INFO_STRING}${OPSTAT}</b>"
	        
	      else
	        STRING="<br />${MOD_DESC[$ID]} ${INFO_STRING}${OPSTAT}"
	    fi
	    
	    
	    
	    CHASSIS_MSG="${CHASSIS_MSG}${STRING}"
	
	  done
	
	
	for ID in $MODULE_ID_LIST
	  do
	
	    unset SERIAL
	    unset NAME
	    [[ "${MOD_SERIAL_NUM[$ID]}" != "" ]] && SERIAL="S/N: ${MOD_SERIAL_NUM[$ID]}"
	    grep -q "Fabric Extender Module" <<< ${MOD_DESC[$ID]} && NAME="${MOD_NAME[$ID]} (${SERIAL}) " || NAME="${MOD_DESC[$ID]} (${MOD_NAME[$ID]} ${SERIAL}) "
	
	    if [[ "${PWR_OPER_STAT[$ID]}" != "on" ]] || [[ "${MOD_OPER_STAT[$ID]}" != "OK" ]]
	      then
	        RETVAL=$STATE_CRITICAL
	        STRING="<br /><b>${NAME} - ${PWR_ADMIN_STAT[$ID]}/${PWR_OPER_STAT[$ID]} - ${MOD_OPER_STAT[$ID]}</b>"
	        
	      else
	        STRING="<br />${NAME} - ${PWR_ADMIN_STAT[$ID]}/${PWR_OPER_STAT[$ID]} - ${MOD_OPER_STAT[$ID]}"
	    fi
	    
	    
	    MODULE_MSG="${MODULE_MSG}${STRING}"
	
	  done
	
	
	
	
	
	RETSTR="Chassis:${CHASSIS_MSG}<br /><br />Module:<br />(Description/Name/Serial - AdminStatus/OperStatus - ModulStatus)${MODULE_MSG}"
	
	
	
	
	
}



if [ $# -lt 1 ];
  then
    echo "Can not find any argument"
    print_usage
    exit $STATE_UNKNOWN
fi


W_LEVEL=""
C_LEVEL=""

while test -n "$1"; do
    case "$1" in
        --help)
            print_help
            exit $STATE_OK
            ;;
        -h)
            print_help
            exit $STATE_OK
            ;;
        --version)
            print_revision $PROGNAME $REVISION
            exit $STATE_OK
            ;;
        -V)
            print_revision $PROGNAME $REVISION
            exit $STATE_OK
            ;;
        -H)
            HOSTADDR=$2
            INTERFACE_IDX_CACHE="${INTERFACE_IDX_CACHE_DIR}/${HOSTADDR}${INTERFACE_IDX_CACHE_EXT}"
            INTERFACE_IDX_LOCK="${INTERFACE_IDX_CACHE_DIR}/${HOSTADDR}${INTERFACE_IDX_LOCK_EXT}"
            shift
            ;;
        -C)
            COMMUNITY=$2
            shift
            ;;
        -w)
            W_LEVEL=$2
            shift
            ;;
        -c)
            C_LEVEL=$2
            shift
            ;;
        -e)
            EXECUTE=$2
            shift
            ;;
        -i)
            IF_DESCR=$2
            shift
            ;;
        -x)
            INTERFACE_IDX_CACHE_MAXAGE=$2
            shift
            ;;
        -E)
            ESCALATION=$2
            [[ "$ESCALATION" != "1" ]] && echo "Incorrect value \"$ESCALATION\"! For activation set -E to \"1\"" && print_usage && exit $STATE_UNKNOWN
            shift
            ;;
        *)
            echo "Unknown argument: *$1*"
            print_usage
            exit $STATE_UNKNOWN
            ;;
    esac
    shift
done



if [[ -z "$HOSTADDR" ]]
  then
    echo "require IP ..."
    exit $STATE_UNKNOWN

elif [[ -z "$COMMUNITY" ]]
  then
    echo "require community-string ..."
    exit $STATE_UNKNOWN

elif [[ -z "$EXECUTE" ]]
  then
    echo "require check-object ..."
    exit $STATE_UNKNOWN

fi


if ! ping -q -c 2 -W 1 $HOSTADDR > /dev/null
  then
    echo "Host not available"
    exit $STATE_UNKNOWN
fi


case "$EXECUTE" in
	check_failover)
		check_failover_status
		;;
	check_memory)
		check_memory_status
		;;
	check_cpu)
		check_cpu_status
		;;
	check_connections)
		check_connections_status
		;;
	check_interface)
		check_interface_counters
		;;
	check_fan)
		check_fan
		;;
	check_power)
		check_power
		;;
	check_temp)
		check_temp
		;;
	check_chassis)
		check_chassis
		;;
	info)
		system_info
		;;
      *)
            echo "Unknown check: *$EXECUTE*"
            exit $STATE_UNKNOWN
            ;;
esac


echo "$RETSTR|$PERF_DATA"

exit $RETVAL
